#!/tools/bin/sh

# This script is run in the chroot environment.  (See build.sh for calling
# conventions.)

set +h
umask 022
export LC_ALL=POSIX
unset CC CXX CPP LD_LIBRARY_PATH LD_PRELOAD

# Make directories

mkdir -p /{bin,boot,dev/{pts,shm},etc/opt,home,lib,mnt,proc} &&
mkdir -p /{root,sbin,tmp,usr/local,var,opt} &&
for dirname in /usr /usr/local
    do
    mkdir $dirname/{bin,etc,include,lib,sbin,share,src} &&
    ln -s share/{man,doc,info} $dirname &&
    mkdir $dirname/share/{dict,doc,info,locale,man} &&
    mkdir $dirname/share/{nls,misc,terminfo,zoneinfo} &&
    mkdir $dirname/share/man/man{1,2,3,4,5,6,7,8}

    if [ $? -ne 0 ]; then exit 1; fi
done
mkdir /var/{lock,log,mail,run,spool} &&
mkdir -p /var/{tmp,opt,cache,lib/misc,local} &&
mkdir /opt/{bin,doc,include,info} &&
mkdir -p /opt/{lib,man/man{1,2,3,4,5,6,7,8}} &&
chmod 0750 /root &&
chmod 1777 /tmp /var/tmp &&
ln -s /tools/bin/{bash,cat,pwd,stty} /bin &&
ln -s /tools/bin/perl /usr/bin &&
ln -s /tools/lib/libgcc_s.so.1 /usr/lib &&
ln -s bash /bin/sh &&
mount proc /proc -t proc &&
mount devpts /dev/pts -t devpts

if [ $? -ne 0 ]; then exit 1; fi

# Create passwd and group files.
cat>/etc/passwd <<EOF &&
root:x:0:0:root:/root:/bin/bash
EOF
cat>/etc/group<<EOF &&
root:x:0:
bin:x:1:
sys:x:2:
kmem:x:3:
tty:x:4:
tape:x:5:
daemon:x:6:
floppy:x:7:
disk:x:8:
lp:x:9:
dialout:x:10:
audio:x:11:
EOF

# Makedev (replace with udev in 2.6).

cd /dev &&
bzcat /sources/MAKEDEV-1.7.bz2 > MAKEDEV-1.7 &&
chmod +x MAKEDEV-1.7 &&
./MAKEDEV-1.7 -v generic-nopty &&
rm MAKEDEV-1.7

if [ $? -ne 0 ]; then exit 1; fi

# Install kernel headers

cd /sources &&
tar xvjf linux-2.4.22.tar.bz2 &&
cd linux-2.4.22 &&
make mrproper &&
make include/linux/version.h &&
make symlinks &&
cp -R include/asm/ /usr/include &&
cp -R include/asm-generic /usr/include &&
cp -R include/linux /usr/include &&
touch /usr/include/linux/autoconf.h
# Don't delete kernel source, we'll compile it later.

if [ $? -ne 0 ]; then exit 1; fi

# glibc

cd /sources &&
tar xvjf glibc-2.3.2.tar.bz2 &&
cd glibc-2.3.2 &&
tar xvjf ../glibc-linuxthreads-2.3.2.tar.bz2 &&
touch /etc/ld.so.conf &&
patch -p1 -i ../glibc-2.3.2-sscanf-1.patch &&
mkdir ../glibc-build &&
cd ../glibc-build &&
../glibc-2.3.2/configure --prefix=/usr --disable-profile --enable-add-ons --libexecdir=/usr/bin --with-headers=/usr/include &&
make &&
make install &&
mkdir -p /usr/lib/locale &&
localedef -i en_US -f ISO-8859-1 en_US &&
# It's broken...
make -C ../glibc-2.3.2/linuxthreads/man &&
make -C ../glibc-2.3.2/linuxthreads/man install &&
cd .. &&
rm -rf glibc-2.3.2 glibc-build &&
cat>/etc/nsswitch.conf<<EOF &&
passwd: files
group: files
shadow: files

publickey: files

hosts: files dns
networks: files

protocols: db files
services: db files
ethers: db files
rpc: db files

netgroup: db files
EOF
ln -fs /usr/share/zoneinfo/America/Chicago /etc/localtime &&
cat>/etc/ld.so.conf<<EOF
# Begin /etc/ld.so.conf

/usr/local/lib
/opt/lib

# End /etc/ld.so.conf
EOF

if [ $? -ne 0 ]; then exit 1; fi

# Tweak gcc spec file to point to new glibc dynamic loader.
sed -i -e 's@ /tools/lib/ld-linux.so.2@ /lib/ld-linux.so.2@g' \
	/tools/lib/gcc-lib/*/*/specs &&

if [ $? -ne 0 ]; then exit 1; fi

# Binutils

cd /sources &&
tar xvjf binutils-2.14.tar.bz2 &&
mkdir binutils-build &&
cd binutils-build &&
../binutils-2.14/configure --prefix=/usr --enable-shared &&
make tooldir=/usr &&
make tooldir=/usr install &&
cp ../binutils-2.14/include/libiberty.h /usr/include &&
cd .. &&
rm -rf binutils-build binutils-2.14

if [ $? -ne 0 ]; then exit 1; fi

# gcc

cd /sources &&
tar xvjf gcc-core-3.3.1.tar.bz2 &&
tar xvjf gcc-g++-3.3.1.tar.bz2 &&
cd gcc-3.3.1 &&
# Do NOT run fixincludes (Replaces no_fixincludes patch from DIR)
grep -v "\./fixinc\.sh" gcc/Makefile.in > tempfile &&
mv tempfile gcc/Makefile.in &&
# Don't install libiberty, binutils already did.
sed -i "s/install:.*install-subdir/install: install-subdir/" libiberty/Makefile.in &&
mkdir ../gcc-build &&
cd ../gcc-build &&
../gcc-3.3.1/configure --prefix=/usr --enable-shared --enable-threads=posix --enable-__cxa_atexit --enable-clocale=gnu --enable-languages=c,c++ &&
make &&
make install &&
ln -s ../usr/bin/cpp /lib &&
ln -s gcc /usr/bin/cc &&
cd .. &&
rm -rf gcc-3.3.1 gcc-build

if [ $? -ne 0 ]; then exit 1; fi

# At this point, the new toolchain is up and running.

#fi

# busybox-1.00

cd /sources &&
tar xvjf busybox-cvs.tar.bz2 &&
cd busybox &&
bzcat ../busybox-config.bz2 > .config &&
make &&
cp busybox /bin &&
make PREFIX=/ install &&
cd .. &&
rm -rf busybox

if [ $? -ne 0 ]; then exit 1; fi

# man-pages-1.60

cd /sources &&
tar xvjf man-pages-1.60.tar.bz2 &&
cd man-pages-1.60 &&
make install &&
cd .. &&
rm -rf man-pages-1.60

if [ $? -ne 0 ]; then exit 1; fi

# coreutils

cd /sources &&
tar xvjf unused/coreutils-5.0.tar.bz2 &&
cd coreutils-5.0 &&
# Don't install inferior hostname, net-tools version is better.
patch -p1 -i ../unused/coreutils-5.0-hostname-2.patch &&
./configure --prefix=/usr &&
make &&
cp src/sort src/comm /bin &&
#cp src/env /usr/bin/env &&
#make install &&
# Is this crappy or what?
#mv /usr/bin/{basename,cat,chgrp,chmod,chown,cp,dd,df} /bin &&
#mv /usr/bin/{dir,dircolors,du,date,echo,false,head} /bin &&
#mv /usr/bin/{install,ln,ls,mkdir,mkfifo,mknod,mv,pwd} /bin &&
#mv /usr/bin/{rm,rmdir,shred,sync,sleep,stty,su,test} /bin &&
#mv /usr/bin/{touch,true,uname,vdir} /bin &&
#mv /usr/bin/chroot /usr/sbin &&
#ln -s test /bin/[ &&
#ln -s ../../bin/install /usr/bin &&
cd .. &&
rm -rf coreutils-5.0

if [ $? -ne 0 ]; then exit 1; fi

# zlib

cd /sources &&
tar xvjf zlib-1.2.1.tar.bz2 &&
cd zlib-1.2.1 &&
./configure --prefix=/usr --shared &&
make &&
make install &&
make clean &&
./configure --prefix=/usr &&
make &&
make install &&
cp zlib.3 /usr/share/man/man3 &&
cd .. &&
rm -rf zlib-1.2.1

if [ $? -ne 0 ]; then exit 1; fi

# findutils

cd /sources &&
tar xvjf unused/findutils-4.1.20.tar.bz2 &&
cd findutils-4.1.20 &&
./configure --prefix=/usr --libexecdir=/usr/bin &&
make &&
#make install &&
cd .. &&
rm -rf findutils-4.1.20

if [ $? -ne 0 ]; then exit 1; fi

# gawk

cd /sources &&
tar xvjf gawk-3.1.3.tar.bz2 &&
cd gawk-3.1.3 &&
patch -p1 -i ../gawk-3.1.3-libexecdir.patch &&
./configure --prefix=/usr --libexecdir=/usr/bin &&
make &&
make install &&
cd .. &&
rm -rf gawk-3.1.3

if [ $? -ne 0 ]; then exit 1; fi

# ncurses-5.3

cd /sources &&
tar xvjf ncurses-5.3.tar.bz2 &&
cd ncurses-5.3 &&
patch -p1 -i ../ncurses-5.3-etip-2.patch &&
./configure --prefix=/usr --with-shared --without-debug &&
make &&
make install &&
chmod 755 /usr/lib/*.5.3 && # Find specifics
chmod 644 /usr/lib/libncurses++.a &&
cd .. &&
rm -rf ncurses-5.3

if [ $? -ne 0 ]; then exit 1; fi

# vim

cd /sources &&
tar xvjf unused/vim-6.2.tar.bz2 &&
cd vim62 &&  # vim is defective.
echo '#define SYS_VIMRC_FILE "/etc/vimrc"' >> src/feature.h
echo '#define SYS_GVIMRC_FILE "/etc/gvimrc"' >> src/feature.h
./configure --prefix=/usr &&
make &&
#make install &&
#ln -s vim /usr/bin/vi &&
#cat>/etc/.vimrc<<EOF &&
#set nocompatible
#set bs=2
#EOF
cd .. &&
rm -rf vim62

if [ $? -ne 0 ]; then exit 1; fi

# m4

cd /sources &&
tar xvjf m4-1.4.tar.bz2 &&
cd m4-1.4 &&
./configure --prefix=/usr &&
make &&
make install &&
cd .. &&
rm -rf m4-1.4

if [ $? -ne 0 ]; then exit 1; fi

# bison

cd /sources &&
tar xvjf bison-1.875.tar.bz2 &&
cd bison-1.875 &&
patch -p1 -i ../bison-1.875-attribute.patch &&
./configure --prefix=/usr &&
make &&
make install &&
cd .. &&
rm -rf bison-1.875

if [ $? -ne 0 ]; then exit 1; fi

# less

cd /sources &&
tar xvjf less-381.tar.bz2 &&
cd less-381 &&
./configure --prefix=/usr --bindir=/bin --sysconfdir=/etc &&
make &&
make install &&
cd .. &&
rm -rf less-381

if [ $? -ne 0 ]; then exit 1; fi

# groff

cd /sources &&
tar xvjf groff-1.19.tar.bz2 &&
cd groff-1.19 &&
PAGE=letter ./configure --prefix=/usr &&
make &&
make install &&
ln -s soelim /usr/bin/zsoelim &&
ln -s eqn /usr/bin/geqn &&
ln -s tbl /usr/bin/gtbl &&
cd .. &&
rm -rf groff-1.19

if [ $? -ne 0 ]; then exit 1; fi

# sed

cd /sources &&
tar xvjf unused/sed-4.0.7.tar.bz2 &&
cd sed-4.0.7 &&
./configure --prefix=/usr --bindir=/bin &&
make &&
#make install &&
cd .. &&
rm -rf sed-4.0.7

if [ $? -ne 0 ]; then exit 1; fi

# flex

cd /sources &&
tar xvjf flex-2.5.4a.tar.bz2 &&
cd flex-2.5.4 &&
./configure --prefix=/usr &&
make &&
make install &&
ln -s libfl.a /usr/lib/libl.a &&
cat>/usr/bin/lex<<EOF &&
#!/bin/sh
exec /usr/bin/flex -l "$@"
EOF
chmod 755 /usr/bin/lex &&
cd .. &&
rm -rf flex-2.5.4

if [ $? -ne 0 ]; then exit 1; fi

# net-tools

cd /sources &&
tar xvjf unused/net-tools-1.60.tar.bz2 &&
cd net-tools-1.60 &&
patch -p1 -i ../unused/net-tools-1.60-miitool-gcc33-1.patch &&
yes "" | make config &&
make &&
#make update &&
cd .. &&
rm -rf net-tools-1.60

if [ $? -ne 0 ]; then exit 1; fi

# inetutils

cd /sources &&
tar xvjf unused/inetutils-1.4.2.tar.bz2 &&
cd inetutils-1.4.2 &&
./configure --prefix=/usr --disable-syslogd --libexecdir=/usr/sbin --disable-logger --sysconfdir=/etc --localstatedir=/var --disable-whois --disable-servers &&
make &&
#make install &&
#mv /usr/bin/ping /bin &&
cd .. &&
rm -rf inetutils-1.4.2

if [ $? -ne 0 ]; then exit 1; fi

# perl

cd /sources &&
tar xvjf perl-5.8.0.tar.bz2 &&
cd perl-5.8.0 &&
./configure.gnu --prefix=/usr &&
make &&
make install &&
cd .. &&
rm -rf perl-5.8.0

if [ $? -ne 0 ]; then exit 1; fi

# I don't care about "info".

#cd /sources &&
#tar xvjf texinfo-4.6.tar.bz2 &&
#./configure --prefix=/usr &&
#make &&
#make install &&
## Note: make TEXMF=/usr/share/texmf install-tex
#cd .. &&
#rm -rf texinfo-4.6.tar.bz2

# autoconf

cd /sources &&
tar xvjf autoconf-2.57.tar.bz2 &&
cd autoconf-2.57 &&
./configure --prefix=/usr &&
make &&
make install &&
cd .. &&
rm -rf autoconf-2.57

if [ $? -ne 0 ]; then exit 1; fi

# automake

cd /sources &&
tar xvjf automake-1.7.6.tar.bz2 &&
cd automake-1.7.6 &&
./configure --prefix=/usr &&
make &&
make install &&
ln -s automake-1.7 /usr/share/automake &&
cd .. &&
rm -rf automake-1.7.6

if [ $? -ne 0 ]; then exit 1; fi

# bash

cd /sources &&
tar xvjf bash-2.05b.tar.bz2 &&
cd bash-2.05b &&
patch -p1 -i ../bash-2.05b-2.patch &&
./configure --prefix=/usr --bindir=/bin &&
make &&
make install &&
cd .. &&
rm -rf bash-2.05b

if [ $? -ne 0 ]; then exit 1; fi

# file

cd /sources &&
tar xvjf unused/file-4.04.tar.bz2 &&
cd file-4.04 &&
./configure --prefix=/usr &&
make &&
#make install &&
cd .. &&
rm -rf file-4.04

if [ $? -ne 0 ]; then exit 1; fi

# libtool (needed by autoconf?)

cd /sources &&
tar xvjf libtool-1.5.tar.bz2 &&
cd libtool-1.5 &&
./configure --prefix=/usr &&
make &&
make install &&
cd .. &&
rm -rf libtool-1.5

if [ $? -ne 0 ]; then exit 1; fi

# bzip2

cd /sources &&
tar xvjf unused/bzip2-1.0.2.tar.bz2 &&
cd bzip2-1.0.2 &&
make -f Makefile-libbz2_so &&
make clean &&
make &&
#make install &&
#cp bzip2-shared /bin/bzip2 &&
#cp -a libbz2.so* /lib &&
#ln -s ../../lib/libbz2.so.1.0 /usr/lib/libbz2.so &&
#rm /usr/bin/{bunzip2,bzcat,bzip2} &&
#mv /usr/bin/{bzip2recover,bzless,bzmore} /bin &&
#ln -s bzip2 /bin/bunzip2 &&
#ln -s bzip2 /bin/bzcat &&
cd .. &&
rm -rf bzip2-1.0.2

if [ $? -ne 0 ]; then exit 1; fi

# diffutils

cd /sources &&
tar xvjf diffutils-2.8.1.tar.bz2 &&
cd diffutils-2.8.1 &&
./configure --prefix=/usr &&
make &&
make install &&
cd .. &&
rm -rf diffutils-2.8.1

if [ $? -ne 0 ]; then exit 1; fi

# gzip/gunzip in busybox need bugs fixed.
rm -f /bin/gzip /bin/gunzip /bin/uncompress

# gzip

cd /sources &&
tar xvjf gzip-1.3.5.tar.bz2 &&
cd gzip-1.3.5 &&
./configure --prefix=/usr &&
sed -i 's%"BINDIR"%/bin%' gzexe.in &&
make &&
make install &&
ln -s /usr/bin/gzip /bin/uncompress &&
cd .. &&
rm -rf gzip-1.3.5

if [ $? -ne 0 ]; then exit 1; fi

# kbd

cd /sources &&
tar xvjf unused/kbd-1.08.tar.bz2 &&
cd kbd-1.08 &&
patch -p1 -i ../unused/kbd-1.08-more-programs.patch &&
./configure &&
make &&
#make install &&
cd .. &&
rm -rf kbd-1.08

if [ $? -ne 0 ]; then exit 1; fi

# e2fsprogs

cd /sources &&
tar xvjf e2fsprogs-1.34.tar.bz2 &&
mkdir e2fsprogs-build &&
cd e2fsprogs-build &&
../e2fsprogs-1.34/configure --prefix=/usr --with-root-prefix="" --enable-elf-shlibs --disable-nls &&
make &&
make install &&
make install-libs &&
cd .. &&
rm -rf e2fsprogs-build e2fsprogs-1.34

if [ $? -ne 0 ]; then exit 1; fi

# grep-2.5.1

cd /sources &&
tar xvjf unused/grep-2.5.1.tar.bz2 &&
cd grep-2.5.1 &&
./configure --prefix=/usr --bindir=/bin --with-included-regex &&
make &&
#make install &&
cd .. &&
rm -rf grep-2.5.1

if [ $? -ne 0 ]; then exit 1; fi

# grub

cd /sources &&
tar xvjf grub-0.93.tar.bz2 &&
cd grub-0.93 &&
patch -p1 -i ../grub-0.93-gcc33-1.patch &&
./configure --prefix=/usr &&
make &&
make install &&
mkdir /boot/grub &&
cp /usr/share/grub/i386-pc/stage{1,2} /boot/grub &&
cd .. &&
rm -rf grub-0.93

if [ $? -ne 0 ]; then exit 1; fi

# man

cd /sources &&
tar xvjf man-1.5m2.tar.bz2 &&
cd man-1.5m2 &&
# Chop unneeded entry out of MANPATH to avoid redundant whois results.
sed -i -e 's/MANPATH.*\/usr\/man/#/' src/man.conf.in &&
# Feed less the -R command
sed -i -e 's/^DEFAULTLESSOPT=.*/DEFAULTLESSOPT="-isR"/' configure &&
# Patch for 80 columns with groff-1.18 and up.
sed -i -e 's#\(sprintf(eos(bufh), "echo \\".\)ll\( %d.%di\\"; ", ll/10, ll%10);\)#\1nr\2#' src/man.c &&
./configure -default -confdir=/etc &&
make &&
make install &&
cd .. &&
rm -rf man-1.5m2

if [ $? -ne 0 ]; then exit 1; fi

# make

cd /sources &&
tar xvjf make-3.80.tar.bz2 &&
cd make-3.80 &&
./configure --prefix=/usr &&
make &&
make install &&
cd .. &&
rm -rf make-3.80

if [ $? -ne 0 ]; then exit 1; fi

# modutils

cd /sources &&
tar xvjf unused/modutils-2.4.25.tar.bz2 &&
cd modutils-2.4.25 &&
./configure &&
make &&
#make install &&
cd .. &&
rm -rf modutils-2.4.25

if [ $? -ne 0 ]; then exit 1; fi

# patch

cd /sources &&
tar xvjf patch-2.5.4.tar.bz2 &&
cd patch-2.5.4 &&
./configure --prefix=/usr &&
make &&
make install &&
cd .. &&
rm -rf patch-2.5.4

if [ $? -ne 0 ]; then exit 1; fi

# procinfo

cd /sources &&
tar xvjf unused/procinfo-18.tar.bz2 &&
cd procinfo-18 &&
make LDLIBS=-lncurses &&
#make install &&
cd .. &&
rm -rf procinfo-18

if [ $? -ne 0 ]; then exit 1; fi

# procps

cd /sources &&
tar xvjf unused/procps-3.1.11.tar.bz2 &&
cd procps-3.1.11 &&
# Fix a locale bug
sed -i -e 's/setlocale(LC_ALL, "");/setlocale(LC_NUMERIC,"C");/' w.c &&
make &&
#make install &&
#rm /lib/libproc.so &&
cd .. &&
rm -rf procps-3.1.11

if [ $? -ne 0 ]; then exit 1; fi

# psmisc

cd /sources &&
tar xvjf unused/psmisc-21.3.tar.bz2 &&
cd psmisc-21.3 &&
./configure --prefix=/usr --exec-prefix=/ &&
make &&
#make install &&
cd .. &&
rm -rf psmisc-21.3

if [ $? -ne 0 ]; then exit 1; fi

# shadow

cd /sources &&
tar xvjf unused/shadow-4.0.3.tar.bz2 &&
cd shadow-4.0.3 &&
touch /var/run/utmp /var/log/{btmp,lastlog,wtmp} &&
chmod 644 /var/run/utmp /var/log/{btmp,lastlog,wtmp} &&
touch /usr/bin/passwd &&
patch -p1 -i ../unused/shadow-4.0.3-newgrp-fix.patch &&
./configure --prefix=/usr --libdir=/usr/lib --enable-shared &&
make &&
#make install &&
# Compensate for completely borked install process
#cp etc/{limits,login.access} /etc &&
#sed -e 's%/var/spool/mail%/var/mail%' -e 's%#MD5_CRYPT_ENAB.no%MD5_CRYPT_ENAB yes%' etc/login.defs.linux > /etc/login.defs &&
#ln -s vipw /usr/bin/vigr &&
#rm /bin/vipw &&
#mv /bin/sg /usr/bin &&
#mv /usr/lib/lib{shadow,misc}.so.0* /lib &&
#ln -sf ../../lib/libshadow.so.0 /usr/lib/libshadow.so &&
#ln -sf ../../lib/libmisc.so.0 /usr/lib/libmisc.so &&
#rm /bin/groups &&
# Enable shadow passwords and groups
#/usr/sbin/pwconv &&
#/usr/sbin/grpconv &&
cd .. &&
rm -rf shadow-4.0.3

if [ $? -ne 0 ]; then exit 1; fi

# sysklogd

cd /sources &&
tar xvjf unused/sysklogd-1.4.1.tar.bz2 &&
cd sysklogd-1.4.1 &&
make &&
#make install &&
cat > /etc/syslog.conf << "EOF" &&
auth,authpriv.* -/var/log/auth.log
*.*;auth,authpriv.none -/var/log/sys.log
daemon.* -/var/log/daemon.log
kern.* -/var/log/kern.log
mail.* -/var/log/mail.log
user.* -/var/log/user.log
*.emerg *
EOF
cd .. &&
rm -rf sysklogd-1.4.1

if [ $? -ne 0 ]; then exit 1; fi

# sysvinit

cd /sources &&
tar xvjf unused/sysvinit-2.85.tar.bz2 &&
cd sysvinit-2.85 &&
make -C src &&
#make -C src install &&
cat > /etc/inittab << "EOF" &&
# Begin /etc/inittab

id:3:initdefault:

si::sysinit:/etc/rc.d/init.d/rc sysinit

l0:0:wait:/etc/rc.d/init.d/rc 0
l1:S1:wait:/etc/rc.d/init.d/rc 1
l2:2:wait:/etc/rc.d/init.d/rc 2
l3:3:wait:/etc/rc.d/init.d/rc 3
l4:4:wait:/etc/rc.d/init.d/rc 4
l5:5:wait:/etc/rc.d/init.d/rc 5
l6:6:wait:/etc/rc.d/init.d/rc 6

ca:12345:ctrlaltdel:/sbin/shutdown -t1 -a -r now

su:S016:once:/sbin/sulogin

1:2345:respawn:/sbin/agetty tty1 9600
2:2345:respawn:/sbin/agetty tty2 9600
3:2345:respawn:/sbin/agetty tty3 9600
4:2345:respawn:/sbin/agetty tty4 9600
5:2345:respawn:/sbin/agetty tty5 9600
6:2345:respawn:/sbin/agetty tty6 9600

# End /etc/inittab
EOF
cd .. &&
rm -rf sysvinit-2.85

if [ $? -ne 0 ]; then exit 1; fi

# tar

cd /sources &&
tar xvjf unused/tar-1.13.25.tar.bz2 &&
cd tar-1.13.25 &&
./configure --prefix=/usr --bindir=/bin --libexecdir=/usr/bin &&
make &&
#make install &&
cd .. &&
rm -rf tar-1.13.25

if [ $? -ne 0 ]; then exit 1; fi

# util-linux

cd /sources &&
tar xvjf unused/util-linux-2.12.tar.bz2 &&
cd util-linux-2.12 &&
# FHS compliance
sed -i 's%etc/adjtime%var/lib/hwclock/adjtime%' hwclock/hwclock.c &&
mkdir -p /var/lib/hwclock &&
./configure &&
make HAVE_SLN=yes &&
#make HAVE_SLN=yes install &&
cd .. &&
rm -rf util-linux-2.12

if [ $? -ne 0 ]; then exit 1; fi

# Misc files
# /etc/services /etc/protocols made from iana-net
# mktemp, tempfile (busybox?)

#install -m 644 /sources/protocols /etc/protocols &&
#install -m 644 /sources/services /etc/services
#
#if [ $? -ne 0 ]; then exit 1; fi


# Note: better /etc/inittab
# /etc/hosts, /etc/resolv.conf
# /etc/fstab
# linux kernel
# grub

# /tools/bin/find /{,usr/,usr/local/}{bin,sbin,lib} -type f -exec /tools/bin/strip --strip-debug '{}' ';
