#!/bin/sh

# todo:
# 	version update on everything
#	swap in kernel headers 2.6 in toolchain
#	uclibc
#	bzcat, bzless...
# Stuff needed:
# Toolchain (binutils, gcc, headers, glibc)
# busybox (replaces coreutils, bzip2, findutils, grep, sed, tar, util-linux, etc.)
#	binutils (ar, strings)
#	coreutils (sort, comm)
#	gawk
#	diffutils
#	gzip
#	bash
#	patch
# make
# ncurses
# miniperl
#  perl needs comm
#
# note: inetutils wants /usr/bin/env at absolute path.  First thing to complain.

# dropped texinfo, gettext

export DIR=`pwd`
set +h
umask 022
export LC_ALL=POSIX
export PATH=/tools/bin:$PATH
unset CC CXX CPP LD_LIBRARY_PATH LD_PRELOAD

########## Phase 1: Make toolchain in /tools ##########

# Create /tools directory.

rm -rf "$DIR/tools"
mkdir -p "$DIR/tools/etc" "$DIR/tools/sources"
rm /tools
ln -s "$DIR/tools" /

# Install (static) binutils in /tools

cd "$DIR/tools/sources" &&
tar xvjf "$DIR/sources/binutils-2.14.tar.bz2" &&
mkdir binutils-build &&
cd binutils-build &&
../binutils-2.14/configure --prefix=/tools --disable-nls &&
make configure-host &&
make LDFLAGS="-all-static" &&
make install
# Don't delete binutils source/build dir, we'll need it in a bit.

if [ $? -ne 0 ]; then exit 1; fi

# Install (static) gcc in /tools

cd "$DIR/tools/sources" &&
tar xvjf "$DIR/sources/gcc-core-3.3.1.tar.bz2" &&
mkdir gcc-build &&
cd gcc-build &&
../gcc-3.3.1/configure --prefix=/tools --with-local-prefix=/tools --disable-nls --enable-shared --enable-languages=c &&
make BOOT_LDFLAGS="-static" bootstrap &&
make install &&
ln -sf gcc "$DIR/tools/bin/cc" &&
cd .. &&
rm -rf gcc-build

if [ $? -ne 0 ]; then exit 1; fi

# Install linux headers in /tools

cd "$DIR/tools/sources" &&
tar xvjf "$DIR/sources/linux-2.4.22.tar.bz2" &&
cd linux-2.4.22 &&
make mrproper &&
make include/linux/version.h &&
make symlinks &&
mkdir "$DIR/tools/include/asm" &&
cp include/asm/* "$DIR/tools/include/asm" &&
cp -R include/asm-generic "$DIR/tools/include" &&
cp -R include/linux "$DIR/tools/include" &&
touch "$DIR/tools/include/linux/autoconf.h" &&
cd .. &&
rm -rf linux-2.4.22

if [ $? -ne 0 ]; then exit 1; fi

# Install glibc in /tools

cd "$DIR/tools/sources" &&
tar xvjf "$DIR/sources/glibc-2.3.2.tar.bz2" &&
cd glibc-2.3.2 &&
tar xvjf "$DIR/sources/glibc-linuxthreads-2.3.2.tar.bz2" &&
touch "$DIR/tools/etc/ld.so.conf" &&
patch -Np1 -i "$DIR/sources/glibc-2.3.2-sscanf-1.patch" &&
mkdir ../glibc-build &&
cd ../glibc-build &&
../glibc-2.3.2/configure --prefix=/tools --disable-profile --enable-add-ons --with-headers=/tools/include --with-binutils=/tools/bin --without-gd &&
make &&
make install &&
mkdir -p "$DIR/tools/lib/locale" &&
localedef -i en_US -f ISO-8859-1 en_US &&
cd .. &&
rm -rf glibc-build glibc-2.3.2

if [ $? -ne 0 ]; then exit 1; fi

# Recompile linker to link against /tools/lib, modify gcc specs file to
# point to new linker, delete leaked headers from parent system

cd "$DIR/tools/sources/binutils-build" &&
make -C ld LDFLAGS="-all-static" LIB_PATH=/tools/lib &&
make -C ld install &&
sed -e 's@ /lib/ld-linux.so.2@ /tools/lib/ld-linux.so.2@g' /tools/lib/gcc-lib/*/*/specs > tempspecfile &&
mv -f tempspecfile "$DIR"/tools/lib/gcc-lib/*/*/specs &&
rm -f "$DIR"/tools/lib/gcc-lib/*/*/include/{pthread.h,bits/sigthread.h} &&
cd .. &&
rm -rf binutils-build

if [ $? -ne 0 ]; then exit 1; fi

# Make sure the toolchain is creating binaries linked against /tools/lib
echo 'main(){}' > dummy.c &&
gcc dummy.c &&
xtest=`readelf -l a.out | grep ': /tools/lib'` &&
rm dummy.c a.out
if [ -z "$xtest" ]
then
  echo Toolchain test 1 failed
  exit 1
fi
unset xtest

# New toolchain in /tools is now functional, containing binutils, gcc, glibc,
# headers, and an adjusted linker.  We can now compile and link things
# with/against the binaries in /tools, which should be able to run
# independently of the host system.

# But we're not _quite_ done yet.  Although the glibc in /tools was compiled
# by the versions of gcc and binutils in /tools, the glibc that those
# gcc and binutils binaries were (statically) linked against was the version
# on the host system.  Also, the binutils was compiled by the host system's
# compiler, not the one in /tools.  More work is needed to purify our
# toolchain and get completely away from the parent system: recompile binutils
# and gcc using the stuff in /tools.

# Rebuild the gcc in /tools, this time dynamically linked.  This time, we
# adjust the source code to know about the /tools directory, rather than
# patching the resulting binaries after the fact.  (This way, /tools is always
# used during the build.)

# Note: the gcc-core source is still around from the last build.  Now add
# the c++ compiler source to it.  (The earlier build we did was in a separate
# directory, which was already deleted.  Nothing was written to the source
# directory during the earlier (static) builds, which is why we didn't delete
# and re-extract the gcc and binutils sources.)

cd "$DIR/tools/sources" &&
tar xvjf "$DIR/sources/gcc-g++-3.3.1.tar.bz2" &&
cd gcc-3.3.1 &&
# Do NOT run fixincludes (Replaces no_fixincludes patch from DIR)
grep -v "\./fixinc\.sh" gcc/Makefile.in > tempfile &&
mv tempfile gcc/Makefile.in &&
# Adjust config entries from which spec file will be generated (so that
# new libraries in /tools used during all phases of build, not just by
# resulting binaries).  (Replaces specs-2 patch from DIR)
for i in `find gcc/config -name "*.h"`
do
  cat "$i" | sed -r -e "s /lib(64)?/ld(64)?(-linux)?(-x86-)?(-ia)?(64)?.so.[12] /tools& " -e "s P,/usr/lib P,/tools/lib " -e "s@-rpath /lib@-rpath /tools/lib@" > tempfile
  if [ $? -ne 0 ]; then exit 1; fi
  cmp -s "$i" tempfile
  if [ $? -ne 0 ]
  then
    chmod 664 tempfile &&
    echo "Changing path in $i" &&
    mv tempfile "$i"
  else
    rm tempfile
  fi
  if [ $? -ne 0 ]; then exit 1; fi
done &&
# Remove /usr/include from the end of the include search path.
# (Also part of specs-2 patch from DIR)
cat >> gcc/config/linux.h << EOF &&
#undef STANDARD_INCLUDE_DIR
#define STANDARD_INCLUDE_DIR 0
EOF
mkdir ../gcc-build &&
cd ../gcc-build &&
../gcc-3.3.1/configure --prefix=/tools --with-local-prefix=/tools --enable-languages=c,c++ --enable-shared --enable-threads=posix --enable-__cxa_atexit --enable-clocale=gnu &&
make &&
make install &&
cd .. &&
# We modified the gcc source, so delete it now.
rm -rf gcc-build gcc-3.3.1

if [ $? -ne 0 ]; then exit 1; fi

# Now rebuild binutils with /tools.  (Binutils source is still around from
# static build.)

cd "$DIR/tools/sources" &&
mkdir binutils-build &&
cd binutils-build &&
../binutils-2.14/configure --prefix=/tools --enable-shared --with-lib-path=/tools/lib &&
make &&
make install &&
# Could do at end of chapter 5:
# make -C ld clean
# make -C ld LIB_PATH=/usr/lib:/lib
# make install
# Does this screw up glibc build?
cd .. &&
rm -rf binutils-build binutils-2.14

if [ $? -ne 0 ]; then exit 1; fi

# busybox-1.0.0

cd "$DIR/tools/sources" &&
tar xvjf "$DIR/sources/busybox-cvs.tar.bz2" &&
cd busybox &&
bzcat "$DIR/sources/busybox-config.bz2" > .config &&
make &&
cp busybox /tools/bin &&
rm /tools/bin/strings && # Binutils stuff we do also
(for i in `sed 's ^.*/  ' busybox.links`; do ln -s busybox /tools/bin/$i; done) &&
# sort awk patch sh
rm /tools/bin/gzip /tools/bin/gunzip &&  # Can't disable this; tar z support...
cd .. &&
rm -rf busybox

if [ $? -ne 0 ]; then exit 1; fi

# Hack out sort from coreutils (temporary measure)

cd "$DIR/tools/sources" &&
tar xvjf "$DIR/sources/coreutils-5.0.tar.bz2" &&
cd coreutils-5.0 &&
./configure --prefix=/tools &&
make &&
strip src/sort -o /tools/bin/sort && #make install &&
cd .. &&
rm -rf coreutils-5.0
if [ $? -ne 0 ]; then exit 1; fi

# make-3.80

cd "$DIR/tools/sources" &&
tar xvjf "$DIR/sources/make-3.80.tar.bz2" &&
cd make-3.80 &&
./configure --prefix=/tools &&
make &&
make install &&
cd .. &&
rm -rf make-3.80

# debug: Temporary measure because busybox awk is horked

# gawk-3.1.3

cd "$DIR/tools/sources" &&
tar xvjf "$DIR/sources/gawk-3.1.3.tar.bz2" &&
cd gawk-3.1.3 &&
./configure --prefix=/tools &&
make &&
make install &&
cd .. && 
rm -rf gawk-3.1.3

if [ $? -ne 0 ]; then exit 1; fi

# Fix this; busybox should have diff (always -u)

# diffutils-2.8.1

cd "$DIR/tools/sources" &&
tar xvjf "$DIR/sources/diffutils-2.8.1.tar.bz2" &&
cd diffutils-2.8.1 &&
./configure --prefix=/tools &&
make &&
make install &&
cd .. &&
rm -rf diffutils-2.8.1

if [ $? -ne 0 ]; then exit 1; fi

# ncurses-5.3

cd "$DIR/tools/sources" &&
tar xvjf "$DIR/sources/ncurses-5.3.tar.bz2" &&
cd ncurses-5.3 &&
patch -p1 -i "$DIR/sources/ncurses-5.3-etip-2.patch" &&
./configure --prefix=/tools --with-shared --without-debug --without-ada --enable-overwrite &&
make &&
make install &&
cd .. &&
rm -rf ncurses-5.3

if [ $? -ne 0 ]; then exit 1; fi

# bash

cd "$DIR/tools/sources" &&
tar xvjf "$DIR/sources/bash-2.05b.tar.bz2" &&
cd bash-2.05b &&
patch -p1 -i "$DIR/sources/bash-2.05b-2.patch" &&
./configure --prefix=/tools &&
make &&
make install &&
cd .. &&
rm -rf bash-2.05b &&
ln -s bash "$DIR/tools/bin/sh"

if [ $? -ne 0 ]; then exit 1; fi

# Debug thingy; busybox gzip and gunzip are broken for more than one file

# gzip-1.3.5

cd "$DIR/tools/sources" &&
tar xvjf "$DIR/sources/gzip-1.3.5.tar.bz2" &&
cd gzip-1.3.5 &&
./configure --prefix=/tools &&
make &&
make install &&
cd .. &&
rm -rf gzip-1.3.5

if [ $? -ne 0 ]; then exit 1; fi

# I skipped gettext, because I just don't care.

# patch

# Note: Power PC needs CPPFLGS=-D_GNU_SOURCE prepended to the configure
# step.  Replace with busybox.

cd "$DIR/tools/sources" &&
tar xvjf "$DIR/sources/patch-2.5.4.tar.bz2" &&
cd patch-2.5.4 &&
./configure --prefix=/tools &&
make &&
make install &&
cd .. &&
rm -rf patch-2.5.4

if [ $? -ne 0 ]; then exit 1; fi

# perl-5.8.0 (microperl)

cd "$DIR/tools/sources" &&
tar xvjf "$DIR/sources/perl-5.8.0.tar.bz2" &&
cd perl-5.8.0 &&
chmod 777 hints/linux.sh &&
sed -i -n 's /lib/ \${prefix}/lib/ ' hints/linux.sh &&
echo 'liconcpth=""' >> hints/linux.sh &&
echo 'loclibpth=""' >> hints/linux.sh &&
echo 'glibpth="${prefix}/lib"' >> hints/linux.sh &&
./configure.gnu --prefix=/tools &&
make -f Makefile.micro &&
cp microperl /tools/bin/perl &&
#make perl utilities &&
#cp perl pod/pod2man "$DIR/tools/bin" &&
#mkdir -p "$DIR/tools/lib/perl5/5.8.0" &&
#cp -R lib/* "$DIR/tools/lib/perl5/5.8.0" &&
cd .. &&
rm -rf perl-5.8.0

if [ $? -ne 0 ]; then exit 1; fi

# strip debugging symbols.

strip --strip-unneeded "$DIR"/tools/{,s}bin/*
strip --strip-debug "$DIR"/tools/lib/*
rm -rf "$DIR"/tools/{,share/}{doc,info,man}

# Get rid of that symlink.
rm /tools

echo "Toolchain build complete."
